@@ -3,7 +3,7 @@
  * 
  * See Copyright for the status of this software.
  * 
- * Author: Aleksey Sanin <aleksey@aleksey.com>
+ * Copyright (C) 2002-2003 Aleksey Sanin <aleksey@aleksey.com>
  */
 #include <stdlib.h>
 #include <string.h>
@@ -16,6 +16,8 @@
 #include <libxml/tree.h>
 #include <libxml/xmlmemory.h>
 #include <libxml/parser.h>
+#include <libxml/xpath.h>
+#include <libxml/xpathInternals.h>
 
 #ifndef XMLSEC_NO_XSLT
 #include <libxslt/xslt.h>
@@ -33,6 +35,7 @@
 #include <xmlsec/transforms.h>
 #include <xmlsec/xmldsig.h>
 #include <xmlsec/xmlenc.h>
+#include <xmlsec/xkms.h>
 #include <xmlsec/parser.h>
 #include <xmlsec/templates.h>
 #include <xmlsec/errors.h>
@@ -51,7 +54,7 @@ static const char bugs[] =
 static const char helpCommands1[] =     
     "Usage: xmlsec <command> [<options>] [<file>]\n"
     "\n"
-    "xmlsec is a command line tool for singining, verifying, encrypting and\n"
+    "xmlsec is a command line tool for signing, verifying, encrypting and\n"
     "decrypting XML documents. The allowed <command> values are:\n"
     "  --help      "	"\tdisplay this help information and exit\n"
     "  --help-all  "	"\tdisplay help information for all commands/options and exit\n"
@@ -71,6 +74,10 @@ static const char helpCommands2[] =
     "  --encrypt   "	"\tencrypt data and output XML document\n"
     "  --decrypt   "	"\tdecrypt data from XML document\n"
 #endif /* XMLSEC_NO_XMLENC */
+#ifndef XMLSEC_NO_XKMS
+    "  --xkis-server-locate  " "\tprocess data as XKMS/XKISS Locate request\n"
+    "  --xkis-server-validate" "\tprocess data as XKMS/XKISS Validate request\n"
+#endif /* XMLSEC_NO_XKMS */
     ;
 
 static const char helpVersion[] = 
@@ -106,6 +113,14 @@ static const char helpDecrypt[] =
     "Usage: xmlsec decrypt [<options>] <file>\n"
     "Decrypts XML Encryption data in the <file>\n";
 
+static const char helpXkissServerLocate[] =  
+    "Usage: xmlsec xkiss-server-locate [<options>] <file>\n"
+    "Processes the <file> as XKMS/XKISS Locate request and outputs the response\n";
+
+static const char helpXkissServerValidate[] =  
+    "Usage: xmlsec xkms-server-validate [<options>] <file>\n"
+    "Processes the <file> as XKMS/XKISS Validate request and outputs the response\n";
+
 static const char helpListKeyData[] =     
     "Usage: xmlsec list-key-data\n"
     "Prints the list of known key data klasses\n";
@@ -121,9 +136,10 @@ static const char helpListTransforms[] =
 #define xmlSecAppCmdLineTopicEncCommon		0x0010
 #define xmlSecAppCmdLineTopicEncEncrypt		0x0020
 #define xmlSecAppCmdLineTopicEncDecrypt		0x0040
-#define xmlSecAppCmdLineTopicKeysMngr		0x0080
-#define xmlSecAppCmdLineTopicX509Certs		0x0100
-#define xmlSecAppCmdLineTopicVersion		0x0200
+#define xmlSecAppCmdLineTopicXkmsCommon		0x0080
+#define xmlSecAppCmdLineTopicKeysMngr		0x1000
+#define xmlSecAppCmdLineTopicX509Certs		0x2000
+#define xmlSecAppCmdLineTopicVersion		0x4000
 #define xmlSecAppCmdLineTopicAll		0xFFFF
 
 /****************************************************************
@@ -142,17 +158,32 @@ static xmlSecAppCmdLineParam helpParam = {
     NULL
 };    
 
+#if !defined(XMLSEC_NO_CRYPTO_DYNAMIC_LOADING) && defined(XMLSEC_CRYPTO_DYNAMIC_LOADING)
+static xmlSecAppCmdLineParam cryptoParam = { 
+    xmlSecAppCmdLineTopicGeneral,
+    "--crypto",
+    NULL,
+    "--crypto <name>"
+    "\n\tthe name of the crypto engine to use (if not specified, the default"
+    "\n\tcrypto engine is used)",
+    xmlSecAppCmdLineParamTypeString,
+    xmlSecAppCmdLineParamFlagNone,
+    NULL
+};    
+#endif /* !defined(XMLSEC_NO_CRYPTO_DYNAMIC_LOADING) && defined(XMLSEC_CRYPTO_DYNAMIC_LOADING) */
+
 static xmlSecAppCmdLineParam cryptoConfigParam = { 
     xmlSecAppCmdLineTopicGeneral,
     "--crypto-config",
     NULL,
-    "--crypto-config"
+    "--crypto-config <path>"
     "\n\tpath to crypto engine configuration",
     xmlSecAppCmdLineParamTypeString,
     xmlSecAppCmdLineParamFlagNone,
     NULL
 };    
 
+
 static xmlSecAppCmdLineParam repeatParam = { 
     xmlSecAppCmdLineTopicGeneral,
     "--repeat",
@@ -181,7 +212,7 @@ static xmlSecAppCmdLineParam printCryptoErrorMsgsParam = {
     "--print-crypto-error-msgs",
     NULL,
     "--print-crypto-error-msgs"
-    "\n\tprint openssl errors stack at the end",
+    "\n\tprint errors stack at the end",
     xmlSecAppCmdLineParamTypeFlag,
     xmlSecAppCmdLineParamFlagNone,
     NULL
@@ -219,9 +250,9 @@ static xmlSecAppCmdLineParam keysFileParam = {
 
 static xmlSecAppCmdLineParam privkeyParam = { 
     xmlSecAppCmdLineTopicKeysMngr,
+    "--privkey-pem",
     "--privkey",
-    NULL,
-    "--privkey[:<name>] <file>[,<cafile>[,<cafile>[...]]]"
+    "--privkey-pem[:<name>] <file>[,<cafile>[,<cafile>[...]]]"
     "\n\tload private key from PEM file and certificates"
     "\n\tthat verify this key",
     xmlSecAppCmdLineParamTypeStringList,
@@ -229,17 +260,65 @@ static xmlSecAppCmdLineParam privkeyParam = {
     NULL
 };
 
+static xmlSecAppCmdLineParam privkeyDerParam = { 
+    xmlSecAppCmdLineTopicKeysMngr,
+    "--privkey-der",
+    NULL,
+    "--privkey-der[:<name>] <file>[,<cafile>[,<cafile>[...]]]"
+    "\n\tload private key from DER file and certificates"
+    "\n\tthat verify this key",
+    xmlSecAppCmdLineParamTypeStringList,
+    xmlSecAppCmdLineParamFlagParamNameValue | xmlSecAppCmdLineParamFlagMultipleValues,
+    NULL
+};
+
+static xmlSecAppCmdLineParam pkcs8PemParam = { 
+    xmlSecAppCmdLineTopicKeysMngr,
+    "--pkcs8-pem",
+    "--privkey-p8-pem",
+    "--pkcs-pem[:<name>] <file>[,<cafile>[,<cafile>[...]]]"
+    "\n\tload private key from PKCS8 PEM file and PEM certificates"
+    "\n\tthat verify this key",
+    xmlSecAppCmdLineParamTypeStringList,
+    xmlSecAppCmdLineParamFlagParamNameValue | xmlSecAppCmdLineParamFlagMultipleValues,
+    NULL
+};
+
+static xmlSecAppCmdLineParam pkcs8DerParam = { 
+    xmlSecAppCmdLineTopicKeysMngr,
+    "--pkcs8-der",
+    "--privkey-p8-der",
+    "--pkcs8-der[:<name>] <file>[,<cafile>[,<cafile>[...]]]"
+    "\n\tload private key from PKCS8 DER file and DER certificates"
+    "\n\tthat verify this key",
+    xmlSecAppCmdLineParamTypeStringList,
+    xmlSecAppCmdLineParamFlagParamNameValue | xmlSecAppCmdLineParamFlagMultipleValues,
+    NULL
+};
+
 static xmlSecAppCmdLineParam pubkeyParam = { 
     xmlSecAppCmdLineTopicKeysMngr,
+    "--pubkey-pem",
     "--pubkey",
-    NULL,
-    "--pubkey[:<name>] <file>"
+    "--pubkey-pem[:<name>] <file>"
     "\n\tload public key from PEM file",
     xmlSecAppCmdLineParamTypeStringList,
     xmlSecAppCmdLineParamFlagParamNameValue | xmlSecAppCmdLineParamFlagMultipleValues,
     NULL
 };
 
+static xmlSecAppCmdLineParam pubkeyDerParam = { 
+    xmlSecAppCmdLineTopicKeysMngr,
+    "--pubkey-der",
+    NULL,
+    "--pubkey-der[:<name>] <file>"
+    "\n\tload public key from DER file",
+    xmlSecAppCmdLineParamTypeStringList,
+    xmlSecAppCmdLineParamFlagParamNameValue | xmlSecAppCmdLineParamFlagMultipleValues,
+    NULL
+};
+
+
 #ifndef XMLSEC_NO_AES    
 static xmlSecAppCmdLineParam aeskeyParam = { 
     xmlSecAppCmdLineTopicKeysMngr,
@@ -335,7 +414,9 @@ static xmlSecAppCmdLineParam sessionKeyParam = {
 };    
 
 static xmlSecAppCmdLineParam outputParam = { 
-    xmlSecAppCmdLineTopicDSigCommon | xmlSecAppCmdLineTopicEncCommon,
+    xmlSecAppCmdLineTopicDSigCommon | 
+    xmlSecAppCmdLineTopicEncCommon | 
+    xmlSecAppCmdLineTopicXkmsCommon,
     "--output",
     "-o",
     "--output <filename>"
@@ -346,7 +427,9 @@ static xmlSecAppCmdLineParam outputParam = {
 };
 
 static xmlSecAppCmdLineParam nodeIdParam = { 
-    xmlSecAppCmdLineTopicDSigCommon | xmlSecAppCmdLineTopicEncCommon,
+    xmlSecAppCmdLineTopicDSigCommon | 
+    xmlSecAppCmdLineTopicEncCommon | 
+    xmlSecAppCmdLineTopicXkmsCommon,
     "--node-id",
     NULL,
     "--node-id <id>"
@@ -357,7 +440,9 @@ static xmlSecAppCmdLineParam nodeIdParam = {
 };    
 
 static xmlSecAppCmdLineParam nodeNameParam = { 
-    xmlSecAppCmdLineTopicDSigCommon | xmlSecAppCmdLineTopicEncCommon,
+    xmlSecAppCmdLineTopicDSigCommon | 
+    xmlSecAppCmdLineTopicEncCommon | 
+    xmlSecAppCmdLineTopicXkmsCommon,
     "--node-name",
     NULL,   
     "--node-name [<namespace-uri>:]<name>"
@@ -369,7 +454,9 @@ static xmlSecAppCmdLineParam nodeNameParam = {
 };    
     
 static xmlSecAppCmdLineParam nodeXPathParam = { 
-    xmlSecAppCmdLineTopicDSigCommon | xmlSecAppCmdLineTopicEncCommon,
+    xmlSecAppCmdLineTopicDSigCommon | 
+    xmlSecAppCmdLineTopicEncCommon | 
+    xmlSecAppCmdLineTopicXkmsCommon,
     "--node-xpath",
     NULL,   
     "--node-xpath <expr>"
@@ -379,9 +466,11 @@ static xmlSecAppCmdLineParam nodeXPathParam = {
     xmlSecAppCmdLineParamFlagNone,
     NULL
 };    
-    
+
 static xmlSecAppCmdLineParam dtdFileParam = { 
-    xmlSecAppCmdLineTopicDSigCommon | xmlSecAppCmdLineTopicEncCommon,
+    xmlSecAppCmdLineTopicDSigCommon | 
+    xmlSecAppCmdLineTopicEncCommon | 
+    xmlSecAppCmdLineTopicXkmsCommon,
     "--dtd-file",
     NULL,   
     "--dtd-file <file>"
@@ -392,7 +481,9 @@ static xmlSecAppCmdLineParam dtdFileParam = {
 };    
 
 static xmlSecAppCmdLineParam printDebugParam = { 
-    xmlSecAppCmdLineTopicDSigCommon | xmlSecAppCmdLineTopicEncCommon,
+    xmlSecAppCmdLineTopicDSigCommon | 
+    xmlSecAppCmdLineTopicEncCommon | 
+    xmlSecAppCmdLineTopicXkmsCommon,
     "--print-debug",
     NULL,   
     "--print-debug"
@@ -403,7 +494,9 @@ static xmlSecAppCmdLineParam printDebugParam = {
 };    
 
 static xmlSecAppCmdLineParam printXmlDebugParam = { 
-    xmlSecAppCmdLineTopicDSigCommon | xmlSecAppCmdLineTopicEncCommon,
+    xmlSecAppCmdLineTopicDSigCommon | 
+    xmlSecAppCmdLineTopicEncCommon | 
+    xmlSecAppCmdLineTopicXkmsCommon,
     "--print-xml-debug",
     NULL,   
     "--print-xml-debug"
@@ -527,11 +620,33 @@ static xmlSecAppCmdLineParam pkcs12Param = {
     NULL
 };
 
+static xmlSecAppCmdLineParam pubkeyCertParam = { 
+    xmlSecAppCmdLineTopicKeysMngr,
+    "--pubkey-cert-pem",
+    "--pubkey-cert",
+    "--pubkey-cert-pem[:<name>] <file>"
+    "\n\tload public key from PEM cert file",
+    xmlSecAppCmdLineParamTypeStringList,
+    xmlSecAppCmdLineParamFlagParamNameValue | xmlSecAppCmdLineParamFlagMultipleValues,
+    NULL
+};
+
+static xmlSecAppCmdLineParam pubkeyCertDerParam = { 
+    xmlSecAppCmdLineTopicKeysMngr,
+    "--pubkey-cert-der",
+    NULL,
+    "--pubkey-cert-der[:<name>] <file>"
+    "\n\tload public key from DER cert file",
+    xmlSecAppCmdLineParamTypeStringList,
+    xmlSecAppCmdLineParamFlagParamNameValue | xmlSecAppCmdLineParamFlagMultipleValues,
+    NULL
+};
+
 static xmlSecAppCmdLineParam trustedParam = { 
     xmlSecAppCmdLineTopicX509Certs,
+    "--trusted-pem",
     "--trusted",
-    NULL,
-    "--trusted <file>"
+    "--trusted-pem <file>"
     "\n\tload trusted (root) certificate from PEM file <file>",
     xmlSecAppCmdLineParamTypeString,
     xmlSecAppCmdLineParamFlagMultipleValues,
@@ -540,9 +655,9 @@ static xmlSecAppCmdLineParam trustedParam = {
 
 static xmlSecAppCmdLineParam untrustedParam = { 
     xmlSecAppCmdLineTopicX509Certs,
+    "--untrusted-pem",
     "--untrusted",
-    NULL,
-    "--untrusted <file>"
+    "--untrusted-pem <file>"
     "\n\tload untrusted certificate from PEM file <file>",
     xmlSecAppCmdLineParamTypeString,
     xmlSecAppCmdLineParamFlagMultipleValues,
@@ -593,6 +708,17 @@ static xmlSecAppCmdLineParam depthParam = {
     xmlSecAppCmdLineParamFlagNone,
     NULL
 };
+
+static xmlSecAppCmdLineParam X509SkipStrictChecksParam = { 
+    xmlSecAppCmdLineTopicX509Certs,
+    "--X509-skip-strict-checks",
+    NULL,    
+    "--X509-skip-strict-checks"
+    "\n\tskip strict checking of X509 data",
+    xmlSecAppCmdLineParamTypeFlag,
+    xmlSecAppCmdLineParamFlagNone,
+    NULL
+};
 #endif /* XMLSEC_NO_X509 */    
 
 static xmlSecAppCmdLineParamPtr parameters[] = {
@@ -627,7 +753,11 @@ static xmlSecAppCmdLineParamPtr parameters[] = {
     &genKeyParam,
     &keysFileParam,
     &privkeyParam,
+    &privkeyDerParam,
+    &pkcs8PemParam,
+    &pkcs8DerParam,
     &pubkeyParam,
+    &pubkeyDerParam,
 #ifndef XMLSEC_NO_AES    
     &aeskeyParam,
 #endif  /* XMLSEC_NO_AES */    
@@ -640,15 +770,21 @@ static xmlSecAppCmdLineParamPtr parameters[] = {
     &pwdParam,
 #ifndef XMLSEC_NO_X509
     &pkcs12Param,
+    &pubkeyCertParam,
+    &pubkeyCertDerParam,
     &trustedParam,
     &untrustedParam,
     &trustedDerParam,
     &untrustedDerParam,
     &verificationTimeParam,
     &depthParam,    
+    &X509SkipStrictChecksParam,    
 #endif /* XMLSEC_NO_X509 */    
     
     /* General configuration params */
+#if !defined(XMLSEC_NO_CRYPTO_DYNAMIC_LOADING) && defined(XMLSEC_CRYPTO_DYNAMIC_LOADING)
+    &cryptoParam,
+#endif /* !defined(XMLSEC_NO_CRYPTO_DYNAMIC_LOADING) && defined(XMLSEC_CRYPTO_DYNAMIC_LOADING) */
     &cryptoConfigParam,
     &repeatParam,
     &disableErrorMsgsParam,
@@ -671,7 +807,9 @@ typedef enum {
     xmlSecAppCommandSignTmpl,
     xmlSecAppCommandEncrypt,
     xmlSecAppCommandDecrypt,
-    xmlSecAppCommandEncryptTmpl
+    xmlSecAppCommandEncryptTmpl,
+    xmlSecAppCommandXkissServerLocate,
+    xmlSecAppCommandXkissServerValidate
 } xmlSecAppCommand;
 
 typedef struct _xmlSecAppXmlData				xmlSecAppXmlData,
@@ -719,6 +857,13 @@ static int			xmlSecAppPrepareEncCtx		(xmlSecEncCtxPtr encCtx);
 static void			xmlSecAppPrintEncCtx		(xmlSecEncCtxPtr encCtx);
 #endif /* XMLSEC_NO_XMLENC */
 
+#ifndef XMLSEC_NO_XKMS
+static int			xmlSecAppXkissServerLocate	(const char* filename);
+static int			xmlSecAppXkissServerValidate	(const char* filename);
+static int			xmlSecAppPrepareXkissServerCtx	(xmlSecXkissServerCtxPtr xkissServerCtx);
+static void			xmlSecAppPrintXkissServerCtx	(xmlSecXkissServerCtxPtr xkissServerCtx);
+#endif /* XMLSEC_NO_XKMS */
+
 static void			xmlSecAppListKeyData		(void);
 static void			xmlSecAppListTransforms		(void);
 
@@ -732,6 +877,8 @@ xmlSecKeysMngrPtr gKeysMngr = NULL;
 int repeats = 1;
 int print_debug = 0;
 clock_t total_time = 0;
+const char* xmlsec_crypto = XMLSEC_CRYPTO;
+const char* tmp = NULL;
 
 int main(int argc, const char **argv) {
     xmlSecAppCmdLineParamTopic cmdLineTopics;
@@ -756,7 +903,7 @@ int main(int argc, const char **argv) {
 	xmlSecAppPrintHelp(subCommand, cmdLineTopics);
 	goto success;
     } else if(command == xmlSecAppCommandVersion) {
-	fprintf(stdout, "%s %s (%s)\n", XMLSEC_PACKAGE, XMLSEC_VERSION, XMLSEC_CRYPTO);
+	fprintf(stdout, "%s %s (%s)\n", PACKAGE, XMLSEC_VERSION, xmlsec_crypto);
 	goto success;
     }
     
@@ -781,6 +928,8 @@ int main(int argc, const char **argv) {
 	case xmlSecAppCommandVerify:
 	case xmlSecAppCommandEncrypt:
 	case xmlSecAppCommandDecrypt:
+	case xmlSecAppCommandXkissServerLocate:
+	case xmlSecAppCommandXkissServerValidate:
 	    if(pos >= argc) {
 		fprintf(stderr, "Error: <file> parameter is requried for this command\n");
 		xmlSecAppPrintUsage();
@@ -792,6 +941,13 @@ int main(int argc, const char **argv) {
     }
     
     /* now init the xmlsec and all other libs */
+#if !defined(XMLSEC_NO_CRYPTO_DYNAMIC_LOADING) && defined(XMLSEC_CRYPTO_DYNAMIC_LOADING)
+    tmp = xmlSecAppCmdLineParamGetString(&cryptoParam);
+    if((tmp != NULL) && (strcmp(tmp, "default") != 0)) {
+	xmlsec_crypto = tmp;
+    }
+#endif /* !defined(XMLSEC_NO_CRYPTO_DYNAMIC_LOADING) && defined(XMLSEC_CRYPTO_DYNAMIC_LOADING) */
+    
     if(xmlSecAppInit() < 0) {
 	fprintf(stderr, "Error: initialization failed\n");
 	xmlSecAppPrintUsage();
@@ -882,6 +1038,25 @@ int main(int argc, const char **argv) {
 	    break;
 #endif /* XMLSEC_NO_TMPL_TEST */
 #endif /* XMLSEC_NO_XMLENC */
+
+#ifndef XMLSEC_NO_XKMS
+	case xmlSecAppCommandXkissServerLocate:
+	    for(i = pos; i < argc; ++i) {
+    	        if(xmlSecAppXkissServerLocate(argv[i]) < 0) {
+		    fprintf(stderr, "Error: failed to process XKISS Locate request from file \"%s\"\n", argv[i]);
+		    goto fail;
+		}
+	    }
+	    break;
+	case xmlSecAppCommandXkissServerValidate:
+	    for(i = pos; i < argc; ++i) {
+    	        if(xmlSecAppXkissServerValidate(argv[i]) < 0) {
+		    fprintf(stderr, "Error: failed to process XKISS Validate request from file \"%s\"\n", argv[i]);
+		    goto fail;
+		}
+	    }
+	    break;
+#endif /* XMLSEC_NO_XKMS */
 	default:
 	    fprintf(stderr, "Error: invalid command %d\n", command);
 	    xmlSecAppPrintUsage();
@@ -1565,6 +1740,153 @@ xmlSecAppPrintEncCtx(xmlSecEncCtxPtr encCtx) {
 
 #endif /* XMLSEC_NO_XMLENC */
 
+#ifndef XMLSEC_NO_XKMS
+static int 
+xmlSecAppXkissServerLocate(const char* filename) {
+    xmlSecAppXmlDataPtr data = NULL;
+    xmlSecXkissServerCtx xkissServerCtx;
+    clock_t start_time;
+    int res = -1;
+
+    if(filename == NULL) {
+	return(-1);
+    }
+
+    if(xmlSecXkissServerCtxInitialize(&xkissServerCtx, gKeysMngr) < 0) {
+	fprintf(stderr, "Error: XKISS server context initialization failed\n");
+	return(-1);
+    }
+    if(xmlSecAppPrepareXkissServerCtx(&xkissServerCtx) < 0) {
+	fprintf(stderr, "Error: XKISS server context preparation failed\n");
+	goto done;
+    }
+
+    /* parse template and select start node */
+    data = xmlSecAppXmlDataCreate(filename, xmlSecNodeLocateRequest, xmlSecXkmsNs);
+    if(data == NULL) {
+	fprintf(stderr, "Error: failed to load template \"%s\"\n", filename);
+	goto done;
+    }
+
+    start_time = clock();          
+    if((xmlSecXkissServerCtxLocate(&xkissServerCtx, data->startNode) < 0) || (xkissServerCtx.result == NULL)) {
+	fprintf(stderr, "Error: failed to process locate request\n");
+	goto done;
+    }
+    total_time += clock() - start_time;    
+    
+    /* print out result only once per execution */
+    if(repeats <= 1) {
+	if(xmlSecAppWriteResult(xkissServerCtx.result, NULL) < 0) {
+	    goto done;
+	}
+    }
+
+    res = 0;    
+
+done:
+    /* print debug info if requested */
+    if(repeats <= 1) { 
+        xmlSecAppPrintXkissServerCtx(&xkissServerCtx);
+    }
+    xmlSecXkissServerCtxFinalize(&xkissServerCtx);
+
+    if(data != NULL) {
+	xmlSecAppXmlDataDestroy(data);
+    }
+    return(res);
+}
+
+static int 
+xmlSecAppXkissServerValidate(const char* filename) {
+    xmlSecAppXmlDataPtr data = NULL;
+    xmlSecXkissServerCtx xkissServerCtx;
+    clock_t start_time;
+    int res = -1;
+
+    if(filename == NULL) {
+	return(-1);
+    }
+
+    if(xmlSecXkissServerCtxInitialize(&xkissServerCtx, gKeysMngr) < 0) {
+	fprintf(stderr, "Error: XKISS server context initialization failed\n");
+	return(-1);
+    }
+    if(xmlSecAppPrepareXkissServerCtx(&xkissServerCtx) < 0) {
+	fprintf(stderr, "Error: XKISS server context preparation failed\n");
+	goto done;
+    }
+
+    /* parse template and select start node */
+    data = xmlSecAppXmlDataCreate(filename, xmlSecNodeValidateRequest, xmlSecXkmsNs);
+    if(data == NULL) {
+	fprintf(stderr, "Error: failed to load template \"%s\"\n", filename);
+	goto done;
+    }
+
+    start_time = clock();          
+    if((xmlSecXkissServerCtxValidate(&xkissServerCtx, data->startNode) < 0) || (xkissServerCtx.result == NULL)) {
+	fprintf(stderr, "Error: failed to process locate request\n");
+	goto done;
+    }
+    total_time += clock() - start_time;    
+    
+    /* print out result only once per execution */
+    if(repeats <= 1) {
+	if(xmlSecAppWriteResult(xkissServerCtx.result, NULL) < 0) {
+	    goto done;
+	}
+    }
+
+    res = 0;    
+
+done:
+    /* print debug info if requested */
+    if(repeats <= 1) { 
+        xmlSecAppPrintXkissServerCtx(&xkissServerCtx);
+    }
+    xmlSecXkissServerCtxFinalize(&xkissServerCtx);
+
+    if(data != NULL) {
+	xmlSecAppXmlDataDestroy(data);
+    }
+    return(res);
+}
+
+static int
+xmlSecAppPrepareXkissServerCtx(xmlSecXkissServerCtxPtr xkissServerCtx) {    
+    if(xkissServerCtx == NULL) {
+	fprintf(stderr, "Error: XKISS  context is null\n");
+	return(-1);
+    }
+
+    /* set key info params */
+    if(xmlSecAppPrepareKeyInfoReadCtx(&(xkissServerCtx->keyInfoReadCtx)) < 0) {
+	fprintf(stderr, "Error: failed to prepare key info context\n");
+	return(-1);
+    }
+
+    return(0);
+}
+
+static void 
+xmlSecAppPrintXkissServerCtx(xmlSecXkissServerCtxPtr xkissServerCtx) {
+    if(xkissServerCtx == NULL) {
+	return;
+    }
+    
+    /* print debug info if requested */
+    if((print_debug != 0) || xmlSecAppCmdLineParamIsSet(&printDebugParam)) {
+	xmlSecXkissServerCtxDebugDump(xkissServerCtx, stdout);
+    }
+    
+    if(xmlSecAppCmdLineParamIsSet(&printXmlDebugParam)) {	   
+	xmlSecXkissServerCtxDebugXmlDump(xkissServerCtx, stdout);
+    }
+}
+
+#endif /* XMLSEC_NO_XKMS */
+
 static void 
 xmlSecAppListKeyData(void) {
     fprintf(stdout, "Registered key data klasses:\n");
@@ -1594,6 +1916,9 @@ xmlSecAppPrepareKeyInfoReadCtx(xmlSecKeyInfoCtxPtr keyInfoCtx) {
     if(xmlSecAppCmdLineParamIsSet(&depthParam)) {
 	keyInfoCtx->certsVerificationDepth = xmlSecAppCmdLineParamGetInt(&depthParam, 0);
     }
+    if(xmlSecAppCmdLineParamIsSet(&X509SkipStrictChecksParam)) {
+	keyInfoCtx->flags |= XMLSEC_KEYINFO_FLAGS_X509DATA_SKIP_STRICT_CHECKS;
+    }
 #endif /* XMLSEC_NO_X509 */
 
     /* read enabled key data list */
@@ -1683,10 +2008,59 @@ xmlSecAppLoadKeys(void) {
 	    fprintf(stderr, "Error: invalid value for option \"%s\".\n", 
 		    privkeyParam.fullName);
 	    return(-1);
-	} else if(xmlSecAppCryptoSimpleKeysMngrPemKeyAndCertsLoad(gKeysMngr, 
+	} else if(xmlSecAppCryptoSimpleKeysMngrKeyAndCertsLoad(gKeysMngr, 
 		    value->strListValue, 
 		    xmlSecAppCmdLineParamGetString(&pwdParam),
-		    value->paramNameValue) < 0) {
+		    value->paramNameValue, 
+		    xmlSecKeyDataFormatPem) < 0) {
+	    fprintf(stderr, "Error: failed to load private key from \"%s\".\n", 
+		    value->strListValue);
+	    return(-1);
+	}
+    }
+
+    for(value = privkeyDerParam.value; value != NULL; value = value->next) {
+	if(value->strValue == NULL) {
+	    fprintf(stderr, "Error: invalid value for option \"%s\".\n", 
+		    privkeyDerParam.fullName);
+	    return(-1);
+	} else if(xmlSecAppCryptoSimpleKeysMngrKeyAndCertsLoad(gKeysMngr, 
+		    value->strListValue, 
+		    xmlSecAppCmdLineParamGetString(&pwdParam),
+		    value->paramNameValue,
+		    xmlSecKeyDataFormatDer) < 0) {
+	    fprintf(stderr, "Error: failed to load private key from \"%s\".\n", 
+		    value->strListValue);
+	    return(-1);
+	}
+    }
+
+    for(value = pkcs8PemParam.value; value != NULL; value = value->next) {
+	if(value->strValue == NULL) {
+	    fprintf(stderr, "Error: invalid value for option \"%s\".\n", 
+		    pkcs8PemParam.fullName);
+	    return(-1);
+	} else if(xmlSecAppCryptoSimpleKeysMngrKeyAndCertsLoad(gKeysMngr, 
+		    value->strListValue, 
+		    xmlSecAppCmdLineParamGetString(&pwdParam),
+		    value->paramNameValue,
+		    xmlSecKeyDataFormatPkcs8Pem) < 0) {
+	    fprintf(stderr, "Error: failed to load private key from \"%s\".\n", 
+		    value->strListValue);
+	    return(-1);
+	}
+    }
+
+    for(value = pkcs8DerParam.value; value != NULL; value = value->next) {
+	if(value->strValue == NULL) {
+	    fprintf(stderr, "Error: invalid value for option \"%s\".\n", 
+		    pkcs8DerParam.fullName);
+	    return(-1);
+	} else if(xmlSecAppCryptoSimpleKeysMngrKeyAndCertsLoad(gKeysMngr, 
+		    value->strListValue, 
+		    xmlSecAppCmdLineParamGetString(&pwdParam),
+		    value->paramNameValue,
+		    xmlSecKeyDataFormatPkcs8Der) < 0) {
 	    fprintf(stderr, "Error: failed to load private key from \"%s\".\n", 
 		    value->strListValue);
 	    return(-1);
@@ -1699,10 +2073,27 @@ xmlSecAppLoadKeys(void) {
 	    fprintf(stderr, "Error: invalid value for option \"%s\".\n", 
 		    pubkeyParam.fullName);
 	    return(-1);
-	} else if(xmlSecAppCryptoSimpleKeysMngrPemKeyAndCertsLoad(gKeysMngr, 
+	} else if(xmlSecAppCryptoSimpleKeysMngrKeyAndCertsLoad(gKeysMngr, 
 		    value->strListValue, 
 		    xmlSecAppCmdLineParamGetString(&pwdParam),
-		    value->paramNameValue) < 0) {
+		    value->paramNameValue,
+		    xmlSecKeyDataFormatPem) < 0) {
+	    fprintf(stderr, "Error: failed to load public key from \"%s\".\n", 
+		    value->strListValue);
+	    return(-1);
+	}
+    }
+
+    for(value = pubkeyDerParam.value; value != NULL; value = value->next) {
+	if(value->strValue == NULL) {
+	    fprintf(stderr, "Error: invalid value for option \"%s\".\n", 
+		    pubkeyDerParam.fullName);
+	    return(-1);
+	} else if(xmlSecAppCryptoSimpleKeysMngrKeyAndCertsLoad(gKeysMngr, 
+		    value->strListValue, 
+		    xmlSecAppCmdLineParamGetString(&pwdParam),
+		    value->paramNameValue,
+		    xmlSecKeyDataFormatDer) < 0) {
 	    fprintf(stderr, "Error: failed to load public key from \"%s\".\n", 
 		    value->strListValue);
 	    return(-1);
@@ -1800,6 +2191,40 @@ xmlSecAppLoadKeys(void) {
     }
 
 
+    /* read all public keys in certs */
+    for(value = pubkeyCertParam.value; value != NULL; value = value->next) {
+	if(value->strValue == NULL) {
+	    fprintf(stderr, "Error: invalid value for option \"%s\".\n", 
+		    pubkeyCertParam.fullName);
+	    return(-1);
+	} else if(xmlSecAppCryptoSimpleKeysMngrKeyAndCertsLoad(gKeysMngr, 
+		    value->strListValue, 
+		    xmlSecAppCmdLineParamGetString(&pwdParam),
+		    value->paramNameValue,
+		    xmlSecKeyDataFormatCertPem) < 0) {
+	    fprintf(stderr, "Error: failed to load public key from \"%s\".\n", 
+		    value->strListValue);
+	    return(-1);
+	}
+    }
+
+    for(value = pubkeyCertDerParam.value; value != NULL; value = value->next) {
+	if(value->strValue == NULL) {
+	    fprintf(stderr, "Error: invalid value for option \"%s\".\n", 
+		    pubkeyCertDerParam.fullName);
+	    return(-1);
+	} else if(xmlSecAppCryptoSimpleKeysMngrKeyAndCertsLoad(gKeysMngr, 
+		    value->strListValue, 
+		    xmlSecAppCmdLineParamGetString(&pwdParam),
+		    value->paramNameValue,
+		    xmlSecKeyDataFormatCertDer) < 0) {
+	    fprintf(stderr, "Error: failed to load public key from \"%s\".\n", 
+		    value->strListValue);
+	    return(-1);
+	}
+    }
+
+
     /* read all untrusted certs */
     for(value = untrustedParam.value; value != NULL; value = value->next) {
 	if(value->strValue == NULL) {
@@ -1852,6 +2277,20 @@ xmlSecAppInit(void) {
 	fprintf(stderr, "Error: xmlsec intialization failed.\n");
 	return(-1);
     }
+    if(xmlSecCheckVersion() != 1) {
+	fprintf(stderr, "Error: loaded xmlsec library version is not compatible.\n");
+	return(-1);
+    }
+
+#if !defined(XMLSEC_NO_CRYPTO_DYNAMIC_LOADING) && defined(XMLSEC_CRYPTO_DYNAMIC_LOADING)
+    if(xmlSecCryptoDLLoadLibrary(BAD_CAST xmlsec_crypto) < 0) {
+	fprintf(stderr, "Error: unable to load xmlsec-%s library. Make sure that you have\n"
+			"this it installed, check shared libraries path (LD_LIBRARY_PATH)\n"
+			"envornment variable or use \"--crypto\" option to specify different\n"
+			"crypto engine.\n", xmlsec_crypto);
+	return(-1);	
+    }
+#endif /* !defined(XMLSEC_NO_CRYPTO_DYNAMIC_LOADING) && defined(XMLSEC_CRYPTO_DYNAMIC_LOADING) */
 
     /* Init Crypto */
     if(xmlSecAppCryptoInit(xmlSecAppCmdLineParamGetString(&cryptoConfigParam)) < 0) {
@@ -1973,6 +2412,16 @@ xmlSecAppXmlDataCreate(const char* filename, const xmlChar* defStartNodeName, co
     } else if(xmlSecAppCmdLineParamGetString(&nodeXPathParam) != NULL) {
 	xmlXPathContextPtr ctx = NULL;
 	xmlXPathObjectPtr obj = NULL;
+	xmlNodePtr rootNode;
+	xmlNsPtr ns;
+	int ret;
+
+	rootNode = xmlDocGetRootElement(data->doc);
+	if(rootNode == NULL) {
+	    fprintf(stderr, "Error: failed to find root node\n"); 
+	    xmlSecAppXmlDataDestroy(data);
+	    return(NULL);    
+	}
 	
 	ctx = xmlXPathNewContext(data->doc);
 	if(ctx == NULL) {
@@ -1981,6 +2430,19 @@ xmlSecAppXmlDataCreate(const char* filename, const xmlChar* defStartNodeName, co
 	    return(NULL);    
 	}
 	
+	/* register namespaces from the root node */
+	for(ns = rootNode->nsDef; ns != NULL; ns = ns->next) {
+	    if(ns->prefix != NULL){
+	        ret = xmlXPathRegisterNs(ctx, ns->prefix, ns->href);
+		if(ret != 0) {
+		    fprintf(stderr, "Error: failed to register namespace \"%s\"\n", ns->prefix);
+		    xmlXPathFreeContext(ctx);
+		    xmlSecAppXmlDataDestroy(data);
+		    return(NULL);    
+		}
+	    }
+	}
+	
 	obj = xmlXPathEval(BAD_CAST xmlSecAppCmdLineParamGetString(&nodeXPathParam), ctx);
 	if(obj == NULL) {
 	    fprintf(stderr, "Error: failed to evaluate xpath expression\n");
@@ -2155,6 +2617,23 @@ xmlSecAppParseCommand(const char* cmd, xmlSecAppCmdLineParamTopic* cmdLineTopics
 #endif /* XMLSEC_NO_TMPL_TEST */
 #endif /* XMLSEC_NO_XMLENC */
 
+#ifndef XMLSEC_NO_XKMS
+    if((strcmp(cmd, "xkiss-server-locate") == 0) || (strcmp(cmd, "--xkiss-server-locate") == 0)) {
+	(*cmdLineTopics) = xmlSecAppCmdLineTopicGeneral |
+			xmlSecAppCmdLineTopicXkmsCommon |
+			xmlSecAppCmdLineTopicKeysMngr |
+			xmlSecAppCmdLineTopicX509Certs;
+	return(xmlSecAppCommandXkissServerLocate);
+    } else 
+    if((strcmp(cmd, "xkiss-server-validate") == 0) || (strcmp(cmd, "--xkiss-server-validate") == 0)) {
+	(*cmdLineTopics) = xmlSecAppCmdLineTopicGeneral |
+			xmlSecAppCmdLineTopicXkmsCommon |
+			xmlSecAppCmdLineTopicKeysMngr |
+			xmlSecAppCmdLineTopicX509Certs;
+	return(xmlSecAppCommandXkissServerValidate);
+    } else
+#endif /* XMLSEC_NO_XKMS */
+
     if(1) {
 	(*cmdLineTopics) = 0;
 	return(xmlSecAppCommandUnknown);
@@ -2198,6 +2677,12 @@ xmlSecAppPrintHelp(xmlSecAppCommand command, xmlSecAppCmdLineParamTopic topics)
     case xmlSecAppCommandEncryptTmpl:
 	fprintf(stdout, "%s\n", helpEncryptTmpl);
         break;
+    case xmlSecAppCommandXkissServerLocate:
+	fprintf(stdout, "%s\n", helpXkissServerLocate);
+        break;
+    case xmlSecAppCommandXkissServerValidate:
+	fprintf(stdout, "%s\n", helpXkissServerValidate);
+        break;
     }
     if(topics != 0) {
 	fprintf(stdout, "Options:\n");
