@@ -6,7 +6,7 @@
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  * 
- * Copyrigth (C) 2002-2003 Aleksey Sanin <aleksey@aleksey.com>
+ * Copyright (C) 2002-2003 Aleksey Sanin <aleksey@aleksey.com>
  */
 #include "globals.h"
 
@@ -108,10 +108,10 @@ xmlSecKeyReqMatchKey(xmlSecKeyReqPtr keyReq, xmlSecKeyPtr key) {
     xmlSecAssert2(keyReq != NULL, -1);
     xmlSecAssert2(xmlSecKeyIsValid(key), -1);
 
-    if((xmlSecKeyGetType(key) & keyReq->keyType) == 0) {
+    if((keyReq->keyType != xmlSecKeyDataTypeUnknown) && ((xmlSecKeyGetType(key) & keyReq->keyType) == 0)) {
 	 return(0);
     }
-    if((keyReq->keyUsage & key->usage) == 0) {
+    if((keyReq->keyUsage != xmlSecKeyDataUsageUnknown) && ((keyReq->keyUsage & key->usage) == 0)) {
 	return(0);
     }
 
@@ -267,7 +267,9 @@ xmlSecKeyCopy(xmlSecKeyPtr keyDst, xmlSecKeyPtr keySrc) {
         }
     }
     
-    keyDst->usage = keySrc->usage;
+    keyDst->usage 	   = keySrc->usage;
+    keyDst->notValidBefore = keySrc->notValidBefore;
+    keyDst->notValidAfter  = keySrc->notValidAfter;
     return(0);
 }
 
@@ -600,7 +602,12 @@ xmlSecKeyDebugDump(xmlSecKeyPtr key, FILE *output) {
     } 
 
     if(key->name != NULL) {
-	fprintf(output, "=== keys name: %s\n", key->name);
+	fprintf(output, "=== key name: %s\n", key->name);
+    }
+    fprintf(output, "=== key usage: %d\n", key->usage);
+    if(key->notValidBefore < key->notValidAfter) {
+        fprintf(output, "=== key not valid before: %ld\n", (unsigned long)key->notValidBefore);
+	fprintf(output, "=== key not valid after: %ld\n", (unsigned long)key->notValidAfter);
     }
     if(key->value != NULL) {
 	xmlSecKeyDataDebugDump(key->value, output);
@@ -643,6 +650,11 @@ xmlSecKeyDebugXmlDump(xmlSecKeyPtr key, FILE *output) {
     if(key->name != NULL) {
 	fprintf(output, "<KeyName>%s</KeyName>\n", key->name);
     }
+    if(key->notValidBefore < key->notValidAfter) {
+        fprintf(output, "<KeyValidity notValidBefore=\"%ld\" notValidAfter=\"%ld\"/>\n",
+		(unsigned long)key->notValidBefore, 
+		(unsigned long)key->notValidAfter);
+    }
 
     if(key->value != NULL) {
 	xmlSecKeyDataDebugXmlDump(key->value, output);
@@ -748,6 +760,66 @@ xmlSecKeyGenerateByName(const xmlChar* name, xmlSecSize sizeBits, xmlSecKeyDataT
     return(xmlSecKeyGenerate(dataId, sizeBits, type));
 }
 
+/**
+ * xmlSecKeyReadBuffer:
+ * @dataId:		the key value data klass.
+ * @buffer:		the buffer that contains the binary data.
+ *
+ * Reads the key value of klass @dataId from a buffer.
+ *
+ * Returns pointer to newly created key or NULL if an error occurs.
+ */
+xmlSecKeyPtr 
+xmlSecKeyReadBuffer(xmlSecKeyDataId dataId, xmlSecBuffer* buffer) {
+    xmlSecKeyInfoCtx keyInfoCtx;
+    xmlSecKeyPtr key;
+    int ret;
+
+    xmlSecAssert2(dataId != xmlSecKeyDataIdUnknown, NULL);
+    xmlSecAssert2(buffer != NULL, NULL);
+
+    /* create key data */
+    key = xmlSecKeyCreate();
+    if(key == NULL) {
+	xmlSecError(XMLSEC_ERRORS_HERE,
+		    xmlSecErrorsSafeString(xmlSecKeyDataKlassGetName(dataId)),
+		    "xmlSecKeyCreate",
+		    XMLSEC_ERRORS_R_XMLSEC_FAILED,
+		    XMLSEC_ERRORS_NO_MESSAGE);
+	return(NULL);    
+    }
+
+    ret = xmlSecKeyInfoCtxInitialize(&keyInfoCtx, NULL);    
+    if(ret < 0) {
+	xmlSecError(XMLSEC_ERRORS_HERE,
+		    xmlSecErrorsSafeString(xmlSecKeyDataKlassGetName(dataId)),
+		    "xmlSecKeyInfoCtxInitialize",
+		    XMLSEC_ERRORS_R_XMLSEC_FAILED,
+		    XMLSEC_ERRORS_NO_MESSAGE);
+	xmlSecKeyDestroy(key);
+	return(NULL);    
+    }
+    
+    keyInfoCtx.keyReq.keyType = xmlSecKeyDataTypeAny;
+    ret = xmlSecKeyDataBinRead(dataId, key, 
+			xmlSecBufferGetData(buffer),
+			xmlSecBufferGetSize(buffer),
+			&keyInfoCtx);	
+    if(ret < 0) {
+	xmlSecError(XMLSEC_ERRORS_HERE,
+		    xmlSecErrorsSafeString(xmlSecKeyDataKlassGetName(dataId)),
+		    "xmlSecKeyDataBinRead",
+		    XMLSEC_ERRORS_R_XMLSEC_FAILED,
+		    XMLSEC_ERRORS_NO_MESSAGE);
+	xmlSecKeyInfoCtxFinalize(&keyInfoCtx);
+	xmlSecKeyDestroy(key);
+	return(NULL);    
+    }
+    xmlSecKeyInfoCtxFinalize(&keyInfoCtx);
+    
+    return(key);
+}
+
 /**
  * xmlSecKeyReadBinaryFile:
  * @dataId:		the key value data klass.
@@ -759,11 +831,8 @@ xmlSecKeyGenerateByName(const xmlChar* name, xmlSecSize sizeBits, xmlSecKeyDataT
  */
 xmlSecKeyPtr 
 xmlSecKeyReadBinaryFile(xmlSecKeyDataId dataId, const char* filename) {
-    xmlSecKeyInfoCtx keyInfoCtx;
-    xmlSecBuffer buffer;
-    xmlSecByte buf[1024];
     xmlSecKeyPtr key;
-    FILE *f;
+    xmlSecBuffer buffer;
     int ret;
     
     xmlSecAssert2(dataId != xmlSecKeyDataIdUnknown, NULL);
@@ -780,82 +849,88 @@ xmlSecKeyReadBinaryFile(xmlSecKeyDataId dataId, const char* filename) {
 	return(NULL);	
     }
 
-    f = fopen(filename, "rb");
-    if(f == NULL) {
+    ret = xmlSecBufferReadFile(&buffer, filename);
+    if(ret < 0) {
 	xmlSecError(XMLSEC_ERRORS_HERE,
 		    xmlSecErrorsSafeString(xmlSecKeyDataKlassGetName(dataId)),
-		    "fopen",
-		    XMLSEC_ERRORS_R_IO_FAILED,
+		    "xmlSecBufferReadFile",
+		    XMLSEC_ERRORS_R_XMLSEC_FAILED,
 		    "filename=%s", 
 		    xmlSecErrorsSafeString(filename));
 	xmlSecBufferFinalize(&buffer);
 	return(NULL);
     }
 
-    while(1) {
-        ret = fread(buf, 1, sizeof(buf), f);
-	if(ret > 0) {
-	    xmlSecBufferAppend(&buffer, buf, ret);
-	} else if(ret == 0) {
-	    break;
-	} else {
-	    xmlSecError(XMLSEC_ERRORS_HERE,
-			xmlSecErrorsSafeString(xmlSecKeyDataKlassGetName(dataId)),
-			"fread",
-			XMLSEC_ERRORS_R_IO_FAILED,
-			"filename=%s", 
-			xmlSecErrorsSafeString(filename));
-	    fclose(f);
-	    xmlSecBufferFinalize(&buffer);
-	    return(NULL);
-	}
-    }
-    fclose(f);    
-    
-    /* create key data */
-    key = xmlSecKeyCreate();
+    key = xmlSecKeyReadBuffer(dataId, &buffer);
     if(key == NULL) {
 	xmlSecError(XMLSEC_ERRORS_HERE,
 		    xmlSecErrorsSafeString(xmlSecKeyDataKlassGetName(dataId)),
-		    "xmlSecKeyCreate",
+		    "xmlSecKeyReadBuffer",
 		    XMLSEC_ERRORS_R_XMLSEC_FAILED,
-		    XMLSEC_ERRORS_NO_MESSAGE);
+		    "filename=%s", 
+		    xmlSecErrorsSafeString(filename));
 	xmlSecBufferFinalize(&buffer);
-	return(NULL);    
+	return(NULL);	
     }
 
-    ret = xmlSecKeyInfoCtxInitialize(&keyInfoCtx, NULL);    
-    if(ret < 0) {
-	xmlSecError(XMLSEC_ERRORS_HERE,
-		    xmlSecErrorsSafeString(xmlSecKeyDataKlassGetName(dataId)),
-		    "xmlSecKeyInfoCtxInitialize",
-		    XMLSEC_ERRORS_R_XMLSEC_FAILED,
-		    XMLSEC_ERRORS_NO_MESSAGE);
-	xmlSecBufferFinalize(&buffer);
-	xmlSecKeyDestroy(key);
-	return(NULL);    
-    }
-    
-    keyInfoCtx.keyReq.keyType = xmlSecKeyDataTypeAny;
-    ret = xmlSecKeyDataBinRead(dataId, key, 
-			xmlSecBufferGetData(&buffer),
-			xmlSecBufferGetSize(&buffer),
-			&keyInfoCtx);	
-    if(ret < 0) {
-	xmlSecError(XMLSEC_ERRORS_HERE,
-		    xmlSecErrorsSafeString(xmlSecKeyDataKlassGetName(dataId)),
-		    "xmlSecKeyDataBinRead",
-		    XMLSEC_ERRORS_R_XMLSEC_FAILED,
-		    XMLSEC_ERRORS_NO_MESSAGE);
-	xmlSecKeyInfoCtxFinalize(&keyInfoCtx);
-	xmlSecBufferFinalize(&buffer);
-	xmlSecKeyDestroy(key);
-	return(NULL);    
-    }
-    xmlSecKeyInfoCtxFinalize(&keyInfoCtx);
     xmlSecBufferFinalize(&buffer);
-    
-    return(key);
+    return (key);
+}
+
+/**
+ * xmlSecKeyReadMemory:
+ * @dataId:		the key value data klass.
+ * @data:		the memory containing the key
+ * @dataSize: 		the size of the memory block
+ *
+ * Reads the key value of klass @dataId from a memory block @data.
+ *
+ * Returns pointer to newly created key or NULL if an error occurs.
+ */
+xmlSecKeyPtr 
+xmlSecKeyReadMemory(xmlSecKeyDataId dataId, const xmlSecByte* data, xmlSecSize dataSize) {
+    xmlSecBuffer buffer;
+    xmlSecKeyPtr key;
+    int ret;
+
+    xmlSecAssert2(dataId != xmlSecKeyDataIdUnknown, NULL);
+    xmlSecAssert2(data != NULL, NULL);
+    xmlSecAssert2(dataSize > 0, NULL);
+
+    /* read file to buffer */
+    ret = xmlSecBufferInitialize(&buffer, 0);
+    if(ret < 0) {
+	xmlSecError(XMLSEC_ERRORS_HERE,
+		    xmlSecErrorsSafeString(xmlSecKeyDataKlassGetName(dataId)),
+		    "xmlSecBufferInitialize",
+		    XMLSEC_ERRORS_R_XMLSEC_FAILED,
+		    XMLSEC_ERRORS_NO_MESSAGE);
+	return(NULL);	
+    }
+
+    if (xmlSecBufferAppend(&buffer, data, dataSize) < 0) {
+	xmlSecError(XMLSEC_ERRORS_HERE,
+		    xmlSecErrorsSafeString(xmlSecKeyDataKlassGetName(dataId)),
+		    "xmlSecBufferAppend",
+		    XMLSEC_ERRORS_R_XMLSEC_FAILED,
+		    XMLSEC_ERRORS_NO_MESSAGE);
+	xmlSecBufferFinalize(&buffer);
+	return(NULL);	
+    }
+
+    key = xmlSecKeyReadBuffer(dataId, &buffer);
+    if(key == NULL) {
+	xmlSecError(XMLSEC_ERRORS_HERE,
+		    xmlSecErrorsSafeString(xmlSecKeyDataKlassGetName(dataId)),
+		    "xmlSecKeyReadBuffer",
+		    XMLSEC_ERRORS_R_XMLSEC_FAILED,
+		    XMLSEC_ERRORS_NO_MESSAGE);
+	xmlSecBufferFinalize(&buffer);
+	return(NULL);	
+    }
+
+    xmlSecBufferFinalize(&buffer);
+    return (key);
 }
 
 /**
@@ -870,7 +945,7 @@ xmlSecKeyReadBinaryFile(xmlSecKeyDataId dataId, const char* filename) {
  */
 xmlSecKeyPtr 		
 xmlSecKeysMngrGetKey(xmlNodePtr keyInfoNode, xmlSecKeyInfoCtxPtr keyInfoCtx) {
-    xmlSecKeyPtr key = NULL;
+    xmlSecKeyPtr key;
     int ret;
     
     xmlSecAssert2(keyInfoCtx != NULL, NULL);
@@ -899,9 +974,11 @@ xmlSecKeysMngrGetKey(xmlNodePtr keyInfoNode, xmlSecKeyInfoCtxPtr keyInfoCtx) {
 	    xmlSecKeyDestroy(key);
 	    return(NULL);
 	}
-	if(xmlSecKeyGetValue(key) != NULL) {
-	    return(key);
-	}
+
+	if((xmlSecKeyGetValue(key) != NULL) &&
+           (xmlSecKeyMatch(key, NULL, &(keyInfoCtx->keyReq)) != 0)) {
+            return(key);
+        }
     }	
     xmlSecKeyDestroy(key);
     
